"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Container = void 0;
var logger_1 = require("./logger");
var Container = (function () {
    function Container(_a) {
        var _b = _a.providers, providers = _b === void 0 ? [] : _b;
        this.instanceMap = new Map();
        this.depsMap = new Map();
        this.providers = new Map();
        this.registerProviders(providers);
    }
    Container.prototype.getInstance = function (name) {
        var _name = typeof name === 'string' ? name : name.name;
        var instance = this.instanceMap.get(_name);
        if (!instance) {
            var def = this.providers.get(_name);
            if (!def) {
                throw new Error("can not find any provider of " + _name);
            }
            else {
                if (def.useClass) {
                    var Construct = def.useClass;
                    instance = new Construct();
                    this.instanceMap.set(_name, instance);
                }
                else if (def.useValue) {
                    instance = def.useValue;
                    this.instanceMap.set(_name, instance);
                }
                else if (def.useFactory) {
                    instance = def.useFactory();
                }
                else {
                    return;
                }
            }
        }
        return instance;
    };
    Container.prototype.registerProviders = function (providers) {
        var _this = this;
        providers.forEach(function (provider) {
            var _provider = provider;
            if (!(provider.useValue ||
                provider.useFactory ||
                provider.useClass)) {
                if (typeof provider !== 'function') {
                    throw new Error('提供的provider不是一个合法的Provider。');
                }
                _provider = {
                    provide: provider.name,
                    useClass: provider,
                };
            }
            var exist = _this.providers.get(_provider.provide);
            if (exist) {
                logger_1.log('注册Provider失败，因为该令牌的Provider已经存在。');
            }
            else {
                _this.providers.set(_provider.provide, _provider);
            }
        });
    };
    Container.prototype.getProviders = function () {
        return this.providers;
    };
    Container.prototype.getInstanceMap = function () {
        return this.instanceMap;
    };
    return Container;
}());
exports.Container = Container;
